﻿using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using System;
using System.Net.Http;
using System.Threading.Tasks;

namespace Falcon.SugarApi.ApiClient
{
    /// <summary>
    /// 通过HttpClient调用api
    /// </summary>
    public class HttpClientHelper:IHttpClientHelper
    {
        /// <summary>
        /// 实例化帮助对象
        /// </summary>
        /// <param name="options">客户端配置</param>
        public HttpClientHelper(IOptions<ApiClientOptions> options) {
            Options = options.Value;
        }

        /// <summary>
        /// 记录日期的ILogger
        /// </summary>
        public ILogger<HttpClientHelper> Log { get; }

        /// <summary>
        /// 客户端配置
        /// </summary>
        public ApiClientOptions Options { get; set; }

        /// <summary>
        /// 向服务器发送POST请求
        /// </summary>
        /// <param name="url">api服务器url地址</param>
        /// <param name="data">要发送的数据</param>
        /// <returns>响应数据</returns>
        /// <exception cref="Falcon.SugarApi.ApiClient.ApiClientException">请求api时候发生的异常</exception>
        public string Post(string url,string data) {
            var result = "";
            using(HttpClient client = new HttpClient()) {
                try {
                    client.Timeout = TimeSpan.FromSeconds(this.Options.ClientTimeout);
                    HttpContent content = new StringContent(data);
                    content.Headers.ContentType = new System.Net.Http.Headers.MediaTypeHeaderValue(Options.MediaType);
                    if(this.Options.KeepAlive) {
                        client.DefaultRequestHeaders.Connection.Add("keep-alive");
                    }
                    Task<HttpResponseMessage> res = client.PostAsync(url,content);
                    if(res.Result.StatusCode == System.Net.HttpStatusCode.OK) {
                        result = res.Result.Content.ReadAsStringAsync().Result;
                    }
                    else {
                        throw new ApiClientException("访问webapi方法状态码错误",url,data,null);
                    }
                }
                catch(Exception ex) {
                    throw new ApiClientException("访问webapi方法异常",url,data,ex);
                }
                finally {
                    client.Dispose();
                }
            }
            return result;
        }

    }
}
