﻿using System;
using System.IO;
using System.Reflection;
using System.Runtime.Loader;

namespace Falcon.SugarApi.Plugin
{
    /// <summary>
    /// 加载程序集帮助方法
    /// </summary>
    public static class AssemblyLoadHelp
    {
        /// <summary>
        /// 加载到当前域程序集
        /// </summary>
        /// <param name="assemblyPath">程序集路径</param>
        /// <returns></returns>
        public static Assembly AssemblyLoad(string assemblyPath) {
            return Assembly.LoadFrom(assemblyPath);
        }
        /// <summary>
        /// 使用ALC加载程序集
        /// </summary>
        /// <param name="assemblyPath">程序集路径</param>
        /// <returns>程序集</returns>
        [Obsolete("方法存在问题，程序集注册的对象无法实现注入，目前未解决。使用AssemblyLoad代替",false)]
        public static Assembly ALCLoad(string assemblyPath) {
            var context = new AssemblyLoadContext(assemblyPath);
            return context.LoadFromAssemblyPath(assemblyPath);
        }

        /// <summary>
        /// 获取程序集的加载上下文AssemblyLoadContext
        /// </summary>
        /// <param name="assembly">程序集</param>
        /// <returns>加载上下文</returns>
        public static AssemblyLoadContext? GetAssemblyLoadContext(Assembly assembly) {
            return AssemblyLoadContext.GetLoadContext(assembly);
        }

        /// <summary>
        /// 使用alc加载程序集到内存，避免程序集文件被锁定
        /// </summary>
        /// <param name="assemblyPath">程序集路径</param>
        /// <returns>程序集</returns>
        public static Assembly ALCLoadInMemory(string assemblyPath) {
            var bys = File.ReadAllBytes(assemblyPath);
            var stream = new MemoryStream(bys);
            var alc = new AssemblyLoadContext(assemblyPath);
            return alc.LoadFromStream(stream);
        }

        /// <summary>
        /// 卸载alc，所有加载的程序集将一同卸载
        /// </summary>
        /// <param name="context">ALC上下文</param>
        public static void ALCUnload(AssemblyLoadContext context) {
            context.Unload();
        }
    }
}
