﻿using System;

namespace Falcon.SugarApi.Proxy
{
    /// <summary>
    /// 对象代理接口
    /// </summary>
    /// <typeparam name="T">代理的对象类型</typeparam>
    public abstract class Proxy<T>:ProxyBase where T:class
    {
        /// <summary>
        /// 通过传入代理对象生成代理
        /// </summary>
        /// <param name="target">要代理的对象</param>
        /// <exception cref="ArgumentNullException">代理对象为空</exception>
        public Proxy(T target):base(target) {}
        /// <summary>
        /// 代理对象
        /// </summary>
        public new T? Target => base.Target as T;
        /// <summary>
        /// 获取对象属性的代理
        /// </summary>
        /// <param name="prop">属性名称</param>
        /// <returns></returns>
        /// <exception cref="PropNotFoundException">没有找到属性</exception>
        public override object? GetFunc(string prop) {
            var p = typeof(T).GetProperty(prop) ?? throw new PropNotFoundException(prop);
            return p.GetValue(this.Target);
        }
        /// <summary>
        /// 设置对象属性的代理
        /// </summary>
        /// <param name="prop">属性名称</param>
        /// <param name="value">属性值</param>
        /// <exception cref="PropNotFoundException">没有找到属性</exception>
        public override void SetAction(string prop,object? value) {
            var p = typeof(T).GetProperty(prop) ?? throw new PropNotFoundException(prop);
            p.SetValue(this.Target,value);
        }
        /// <summary>
        /// 执行代理对象方法
        /// </summary>
        /// <param name="name">方法名称</param>
        /// <param name="args">传入的参数</param>
        /// <returns>方法返回值</returns>
        /// <exception cref="MethodNotFoundException">没有找到方法</exception>
        public override object? Invoke(string name,params object[] args) {
            var m = typeof(T).GetMethod(name) ?? throw new MethodNotFoundException(name);
            return m.Invoke(this.Target,args);
        }
    }
}
